import MuseScore 3.0
import QtQuick 2.0

MuseScore {

    version: "1.0";
    description: "This demo plugin adds new chords\n\n" +
                 "Comments, feedbacks, report bugs, ask for new features, contribute:\n" +
                 "https://github.com/rousselmanu/msc_plugins/";
    menuPath: "Plugins.Chords." + qsTr("Add Chord Demo")
       
    
   
    // create and return a new Note element with given pitch, tpc
	function createNote(pitch, tpc){
		console.log("pitch = ", pitch, "tpc = ", tpc)
        var note = newElement(Element.NOTE);
        note.pitch = pitch;
		note.tpc = tpc;
        note.tpc1 = tpc;
        note.tpc2 = tpc;
        return note;
    }
    
    function setCursorToTime(cursor, time){
        cursor.rewind(0);
        while (cursor.segment) { 
            var current_time = cursor.tick;
            if(current_time>=time){
                return true;
            }
            cursor.next();
        }
        cursor.rewind(0);
        return false;
    }
    //adds chord at current position. chords is an array with pitch of notes.
	function addChord(cursor, chords, duration){ 
		for (var c = 0; c < chords.length; c++){
			console.log("chords = ", chords[c]);
		}
		if(chords.length==0) return -1;
		var cur_time=cursor.tick;
		cursor.setDuration(duration, 1920);
		cursor.addNote(chords[0]); //add 1st note
		
		var next_time=cursor.tick;
		setCursorToTime(cursor, cur_time); //rewind to this note
		var note = cursor.element.notes[0];
		note.tpc = chords[4];
		
		var chord = cursor.element; //get the chord created when 1st note was inserted
				
		for(var i=1; i<4; i++){
			var note = createNote(chords[i], chords[i+4]);
			console.log("note.pitch = ", note.pitch, "note.tpc = ", note.tpc);
			note.tpc = chords[i+4];
			chord.add(note); //add notes to the chord
			chord.notes[i].tpc = chords[i+4];
		}
		
		setCursorToTime(cursor, next_time);
		return 0;
	}
    
    onRun: {
        if (typeof curScore === 'undefined') {
            Qt.quit();
        }

        var cursor = curScore.newCursor(),
                startStaff,
                endStaff,
                endTick,
                fullScore = false;
                
        cursor.rewind(1);
        

        if (!cursor.segment) { // no selection
            fullScore = true;
            startStaff = 0; // start with 1st staff
            endStaff = curScore.nstaves - 1; // and end with last
        } else {
            startStaff = cursor.staffIdx;
            cursor.rewind(2);
            if (cursor.tick === 0) {
                // this happens when the selection includes
                // the last measure of the score.
                // rewind(2) goes behind the last segment (where
                // there's none) and sets tick=0
                endTick = curScore.lastSegment.tick + 1;
            } else {
                endTick = cursor.tick;
            }
            endStaff = cursor.staffIdx;
        }
        console.log(startStaff + " - " + endStaff + " - " + endTick);
        console.log('curScore.nstaves: ' + curScore.nstaves);
        console.log('cursor.tick: ' + cursor.tick);
        console.log('curScore.lastSegment.tick: ' + curScore.lastSegment.tick);

        cursor.rewind(1); // beginning of selection
        if (fullScore) { // no selection
            cursor.rewind(0); // beginning of score
        }
        cursor.voice = 0;
        
		console.log('cursor.staffIdx: ' + cursor.staffIdx);
		
        //cursor.track = 0;
        
		
       
		
		cursor.staffIdx = 1;
        var chords = [
["Am7", [55,59,60,64,15,19,14,18]],
["Am7", [57,60,64,67,17,14,18,15]],
["Am7#5", [55,57,60,65,15,17,14,13]],
["Am9", [48,52,55,59,14,18,15,19]],
["Bdim9", [59,60,62,68,19,14,16,10]],
["Bm7b5", [57,59,62,65,17,19,16,13]],
["Bm7b5", [59,62,65,69,19,16,13,17]],
["Bm7b5", [59,64,65,69,19,18,13,17]],
["Bm9b5", [59,60,65,69,19,14,13,17]],
["C", [59,60,64,67,19,14,18,15]],
["C/C", [48,52,55,59,14,18,15,19]],
["C69", [52,55,57,62,18,15,17,16]],
["C69", [52,57,59,62,18,17,19,16]],
["Cdim", [48,51,54,57,14,11,20,17]],
["CM#9", [52,55,60,63,18,15,14,11]],
["CM(#5)", [48,52,56,59,14,18,22,19]],
["CM9", [48,50,55,59,14,16,15,19]],
["CM9", [59,60,62,67,19,14,16,15]],
["CM9", [52,55,59,62,18,15,19,16]],
["Cmm9", [48,50,51,59,14,16,11,19]],
["CMsus#11", [48,53,54,59,14,13,8,19]],
["CMsusb13", [48,53,56,59,14,13,10,19]],
["Db7alt", [53,57,59,64,13,17,19,18]],
["Db7alt", [59,64,65,69,19,18,13,17]],
["Dm11", [50,53,55,60,16,13,15,14]],
["Dm13", [50,53,59,60,16,13,19,14]],
["Dm9", [50,52,53,60,16,18,13,14]],
["Dm9", [50,52,57,60,16,18,17,14]],
["DmMaj7", [49,50,53,57,21,16,13,17]],
["DmMaj7", [50,53,57,61,16,13,17,21]],
["DmMaj7", [57,61,62,65,17,21,16,13]],
["DmMaj9", [53,57,61,64,13,17,21,18]],
["E13sus", [52,57,60,62,18,17,14,16]],
["Em9", [52,53,55,62,18,13,15,16]],
["F7#11", [53,59,60,63,13,19,14,11]],
["FM#4", [53,57,59,60,13,17,19,14]],
["FM13", [53,60,62,64,13,14,16,18]],
["FM9", [53,55,57,64,13,15,17,18]],
["G13", [55,62,64,65,15,16,18,13]],
["G13", [53,59,62,64,13,19,16,18]],
["G13", [53,55,59,64,13,15,19,18]],
["G13", [55,59,64,65,15,19,18,13]],
["G13sus", [55,60,64,65,15,14,18,13]],
["G7", [53,57,59,64,13,17,19,18]],
["G7", [59,64,65,69,19,18,13,17]],
["G7#11", [47,49,53,57,19,21,13,17]],
["G7#11", [55,59,61,65,15,19,21,13]],
["G7#9", [59,65,67,70,19,13,15,12]],
["G7#9", [53,58,59,62,13,12,19,16]],
["G7/G", [55,59,62,65,15,19,16,13]],
["G7b13", [55,59,63,65,15,19,11,13]],
["G7b13", [53,57,59,63,13,17,19,23]],
["G7b9", [55,56,59,65,15,10,19,13]],
["G7b9", [59,64,65,68,19,18,13,10]],
["G7sus", [55,60,62,65,15,14,16,13]],
["G7sus", [50,53,55,60,16,13,15,14]],
["G7sus", [48,50,53,55,14,16,13,15]],
["G7sus", [55,59,60,65,15,19,14,13]],
["G7susb9", [55,56,60,65,15,10,14,13]],
["G9", [55,57,59,65,15,17,19,13]],
["G9sus", [55,57,60,65,15,17,14,13]],
["GMsus", [55,59,60,66,15,19,14,20]],
];

		for (var c = 0; c < chords.length; c++){
			cursor.staffIdx = 0;
			var harmony = newElement(Element.HARMONY);
            harmony.text = chords[c][0];
            cursor.add(harmony);
			cursor.staffIdx = 1;
			var chord = chords[c][1];
			addChord(cursor, chord, 1920);
			
		}
		Qt.quit();
    } // end onRun
}
